import { computed, isReactive, isRef, reactive, ref, unref } from 'vue';
import { isArray, isObject, isDefined } from '../common';
import { deepCopy } from '../common/deep-copy/deep-copy';
export class FormField {
    constructor(name, modelValue, rules) {
        this.rulesValidating = ref(0);
        this.touched = false;
        this.errors = computed(() => this._errors.filter(isDefined));
        this.validating = computed(() => this.rulesValidating.value > 0);
        this.hasError = computed(() => this.errors.value.length > 0);
        this.name = name;
        this._errors = reactive(rules.map(() => null));
        if (isRef(modelValue) || isReactive(modelValue)) {
            this.modelValue = modelValue;
            this._initialModelValue = deepCopy(unref(modelValue));
        }
        else if (isObject(modelValue)) {
            this.modelValue = reactive(modelValue);
            this._initialModelValue = deepCopy(this.modelValue);
        }
        else {
            this.modelValue = ref(modelValue);
            this._initialModelValue = deepCopy(unref(modelValue));
        }
    }
    setError(ruleNumber, error) {
        this._errors[ruleNumber] = error;
    }
    reset(toDefaultValues) {
        this.touched = false;
        if (toDefaultValues) {
            if (isRef(this.modelValue)) {
                if (isArray(this.modelValue.value)) {
                    this.modelValue.value = deepCopy(this._initialModelValue);
                }
                else {
                    this.modelValue.value = this._initialModelValue;
                }
            }
            else {
                const copy = deepCopy(this._initialModelValue);
                Object.assign(this.modelValue, copy);
            }
        }
        Object.assign(this._errors, this._errors.map(() => null));
    }
}
